@@ -30,6 +30,13 @@ unsigned long *mcount_arch_parent_location(struct uftrace_sym_info *symtabs,
 #define ARCH_PLT0_SIZE 20
 #define ARCH_PLTHOOK_ADDR_OFFSET 0
 
+/* index of module-ID in the PLTGOT table */
+#define ARCH_PLTGOT_MOD_ID 1
+/* index of resolver address in the PLTGOT table */
+#define ARCH_PLTGOT_RESOLVE 2
+/* number of reserved entries in the PLTGOT table */
+#define ARCH_PLTGOT_OFFSET 3
+
 #define NOP_INSN_SIZE 4
 
 #endif /* MCOUNT_ARCH_H */
