@@ -28,6 +28,13 @@ unsigned long *mcount_arch_parent_location(struct uftrace_sym_info *symtabs,
 #define ARCH_PLT0_SIZE 16
 #define ARCH_PLTHOOK_ADDR_OFFSET 6
 
+/* index of module-ID in the PLTGOT table */
+#define ARCH_PLTGOT_MOD_ID 1
+/* index of resolver address in the PLTGOT table */
+#define ARCH_PLTGOT_RESOLVE 2
+/* number of reserved entries in the PLTGOT table */
+#define ARCH_PLTGOT_OFFSET 3
+
 #define ARCH_CAN_RESTORE_PLTHOOK 1
 
 #define CALL_INSN_SIZE 5
