@@ -12,6 +12,10 @@
 #include "utils/symbol.h"
 #include "utils/utils.h"
 
+#ifndef MAP_FIXED_NOREPLACE
+#define MAP_FIXED_NOREPLACE MAP_FIXED
+#endif
+
 static const unsigned char fentry_nop_patt1[] = { 0x67, 0x0f, 0x1f, 0x04, 0x00 };
 static const unsigned char fentry_nop_patt2[] = { 0x0f, 0x1f, 0x44, 0x00, 0x00 };
 static const unsigned char patchable_gcc_nop[] = { 0x90, 0x90, 0x90, 0x90, 0x90 };
@@ -43,10 +47,12 @@ int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
 
 		trampoline_check = mmap((void *)mdi->trampoline, PAGE_SIZE,
 					PROT_READ | PROT_WRITE | PROT_EXEC,
-					MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+					MAP_FIXED_NOREPLACE | MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 
-		if (trampoline_check == MAP_FAILED)
-			pr_err("failed to mmap trampoline for setup");
+		if (trampoline_check != (void *)mdi->trampoline) {
+			pr_err("could not map trampoline at desired location %#lx, got %#lx: %m\n",
+			       mdi->trampoline, (uintptr_t)trampoline_check);
+		}
 	}
 
 	if (mprotect(PAGE_ADDR(mdi->text_addr), PAGE_LEN(mdi->text_addr, mdi->text_size),
@@ -623,6 +629,7 @@ int mcount_unpatch_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *
 
 	switch (mdi->type) {
 	case DYNAMIC_FENTRY:
+	case DYNAMIC_PATCHABLE:
 		result = unpatch_fentry_func(mdi, sym);
 		break;
 
