@@ -5,13 +5,12 @@
 #include <sys/stat.h>
 #include <time.h>
 
-#include "libtraceevent/event-parse.h"
-#include "libtraceevent/kbuffer.h"
 #include "uftrace.h"
 #include "utils/event.h"
 #include "utils/filter.h"
 #include "utils/fstack.h"
 #include "utils/graph.h"
+#include "utils/kernel-parser.h"
 #include "utils/kernel.h"
 #include "utils/list.h"
 #include "utils/utils.h"
@@ -657,34 +656,36 @@ static void dump_raw_cpu_start(struct uftrace_dump_ops *ops, struct uftrace_kern
 			       int cpu)
 {
 	struct uftrace_raw_dump *raw = container_of(ops, typeof(*raw), ops);
-	struct kbuffer *kbuf = kernel->kbufs[cpu];
 
 	pr_out("reading kernel-cpu%d.dat\n", cpu);
 
 	raw->file_offset = 0;
-	raw->kbuf_offset = kbuffer_curr_offset(kbuf);
+	raw->kbuf_offset = __kparser_curr_offset(&kernel->parser, cpu);
 }
 
+/* internal kernel tracing ring buffer format for extended timestamp */
+#define KERNEL_TRACING_TIME_EXTEND 30
+
 static void dump_raw_kernel_rstack(struct uftrace_dump_ops *ops,
 				   struct uftrace_kernel_reader *kernel, int cpu,
 				   struct uftrace_record *frs, char *name)
 {
 	int tid = kernel->tids[cpu];
-	struct kbuffer *kbuf = kernel->kbufs[cpu];
+	struct uftrace_kernel_parser *kp = &kernel->parser;
 	struct uftrace_raw_dump *raw = container_of(ops, typeof(*raw), ops);
 
 	/* check dummy 'time extend' record at the beginning */
 	if (raw->kbuf_offset == 0x18) {
 		uint64_t offset = 0x10;
 		uint64_t timestamp = 0;
-		void *data = kbuffer_read_at_offset(kbuf, offset, NULL);
+		void *data = __kparser_read_offset(kp, cpu, offset);
 		unsigned char *tmp = data - 12; /* data still returns next record */
 
-		if ((*tmp & 0x1f) == KBUFFER_TYPE_TIME_EXTEND) {
+		if ((*tmp & 0x1f) == KERNEL_TRACING_TIME_EXTEND) {
 			uint32_t upper, lower;
 			int size;
 
-			size = kbuffer_event_size(kbuf);
+			size = __kparser_event_size(kp, cpu);
 
 			memcpy(&lower, tmp, 4);
 			memcpy(&upper, tmp + 4, 4);
@@ -696,7 +697,7 @@ static void dump_raw_kernel_rstack(struct uftrace_dump_ops *ops,
 
 			if (debug)
 				pr_hex(&offset, tmp, 8);
-			else if (kbuffer_next_event(kbuf, NULL))
+			else if (__kparser_next_event(kp, cpu))
 				raw->kbuf_offset += size + 4; // 4 = event header size
 			else
 				raw->kbuf_offset = 0;
@@ -709,14 +710,14 @@ static void dump_raw_kernel_rstack(struct uftrace_dump_ops *ops,
 
 	if (debug) {
 		/* this is only needed for hex dump */
-		void *data = kbuffer_read_at_offset(kbuf, raw->kbuf_offset, NULL);
+		void *data = __kparser_read_offset(kp, cpu, raw->kbuf_offset);
 		int size;
 
-		size = kbuffer_event_size(kbuf);
-		raw->file_offset = kernel->offsets[cpu] + kbuffer_curr_offset(kbuf);
+		size = __kparser_event_size(kp, cpu);
+		raw->file_offset = __kparser_curr_offset(kp, cpu);
 		pr_hex(&raw->file_offset, data - 4, size + 4);
 
-		if (kbuffer_next_event(kbuf, NULL))
+		if (__kparser_next_event(kp, cpu))
 			raw->kbuf_offset += size + 4; // 4 = event header size
 		else
 			raw->kbuf_offset = 0;
@@ -728,32 +729,33 @@ static void dump_raw_kernel_event(struct uftrace_dump_ops *ops,
 				  struct uftrace_record *frs)
 {
 	struct uftrace_raw_dump *raw = container_of(ops, typeof(*raw), ops);
-	struct event_format *event;
+	struct uftrace_kernel_parser *kp = &kernel->parser;
 	int tid = kernel->tids[cpu];
+	void *event;
 	char *event_data;
+	char buf[512];
 	int size = 0;
 
-	event = pevent_find_event(kernel->pevent, frs->addr);
+	event = kparser_find_event(kp, frs->addr);
 	if (!event)
 		return;
 
+	kparser_event_name(kp, event, buf, sizeof(buf));
 	event_data = read_kernel_event(kernel, cpu, &size);
 
 	pr_time(frs->time);
-	pr_out("%5d: [%s] %s:%s(%ld) %.*s\n", tid, rstack_type(frs), event->system, event->name,
-	       frs->addr, size, event_data);
+	pr_out("%5d: [%s] %s(%ld) %.*s\n", tid, rstack_type(frs), buf, frs->addr, size, event_data);
 
 	if (debug) {
 		/* this is only needed for hex dump */
-		struct kbuffer *kbuf = kernel->kbufs[cpu];
-		void *data = kbuffer_read_at_offset(kbuf, raw->kbuf_offset, NULL);
+		void *data = __kparser_read_offset(kp, cpu, raw->kbuf_offset);
 		int size;
 
-		size = kbuffer_event_size(kbuf);
-		raw->file_offset = kernel->offsets[cpu] + kbuffer_curr_offset(kbuf);
+		size = __kparser_event_size(kp, cpu);
+		raw->file_offset = __kparser_curr_offset(kp, cpu);
 		pr_hex(&raw->file_offset, data - 4, size + 4);
 
-		if (kbuffer_next_event(kbuf, NULL))
+		if (__kparser_next_event(kp, cpu))
 			raw->kbuf_offset += size + 4; // 4 = event header size
 		else
 			raw->kbuf_offset = 0;
@@ -1354,7 +1356,7 @@ static void dump_mermaid_footer(struct uftrace_dump_ops *ops, struct uftrace_dat
 	pr_out("flowchart TB\n");
 	print_graph_node_mermaid(&mermaid_graph, &mermaid_graph.root);
 	pr_out("</div>\n");
-	pr_out("<script src=\"https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js\"></script>\n");
+	pr_out("<script src=\"https://cdn.jsdelivr.net/npm/mermaid@9/dist/mermaid.min.js\"></script>\n");
 	pr_out("<script>\n");
 	pr_out("var config = {\n");
 	pr_out("	startOnLoad: true,\n");
@@ -1449,25 +1451,22 @@ static void do_dump_file(struct uftrace_dump_ops *ops, struct uftrace_opts *opts
 		struct uftrace_kernel_reader *kernel = handle->kernel;
 		struct uftrace_record *frs = &kernel->rstacks[i];
 		struct uftrace_session *fsess = handle->sessions.first;
-		struct stat statbuf;
 
-		if (fstat(kernel->fds[i], &statbuf) < 0)
-			continue;
-		if (statbuf.st_size == 0)
+		if (kparser_data_size(&kernel->parser, i) == 0)
 			continue;
 
 		call_if_nonull(ops->cpu_start, ops, kernel, i);
 
 		while (!read_kernel_cpu_data(kernel, i) && !uftrace_done) {
 			int tid = kernel->tids[i];
-			int losts = kernel->missed_events[i];
+			int losts = kparser_missed_events(&kernel->parser, i);
 			struct uftrace_symbol *sym = NULL;
 			char *name;
 			uint64_t addr;
 
 			if (losts) {
 				call_if_nonull(ops->lost, ops, frs->time, tid, losts);
-				kernel->missed_events[i] = 0;
+				kparser_clear_missed(&kernel->parser, i);
 			}
 
 			if (!check_time_range(&handle->time_range, frs->time))
@@ -1810,3 +1809,68 @@ int command_dump(int argc, char *argv[], struct uftrace_opts *opts)
 
 	return ret;
 }
+
+#ifdef UNIT_TEST
+TEST_CASE(dump_command1)
+{
+	struct uftrace_opts opts = {
+		.dirname = "dump-file-test",
+		.exename = read_exename(),
+		.max_stack = 10,
+		.depth = OPT_DEPTH_DEFAULT,
+	};
+	struct uftrace_data handle;
+	struct uftrace_raw_dump dump = {
+		.ops = {
+			.header         = dump_raw_header,
+			.task_start     = dump_raw_task_start,
+			.inverted_time  = dump_raw_inverted_time,
+			.task_rstack    = dump_raw_task_rstack,
+			.task_event     = dump_raw_task_event,
+			.kernel_start   = dump_raw_kernel_start,
+			.cpu_start      = dump_raw_cpu_start,
+			.kernel_func    = dump_raw_kernel_rstack,
+			.kernel_event   = dump_raw_kernel_event,
+			.lost           = dump_raw_kernel_lost,
+			.perf_start     = dump_raw_perf_start,
+			.perf_event     = dump_raw_perf_event,
+		},
+	};
+
+	TEST_EQ(prepare_test_data(&opts, &handle), 0);
+
+	pr_dbg("dump each file contents\n");
+	do_dump_file(&dump.ops, &opts, &handle);
+
+	release_test_data(&opts, &handle);
+	return TEST_OK;
+}
+
+TEST_CASE(dump_command2)
+{
+	struct uftrace_opts opts = {
+		.dirname = "dump-replay-test",
+		.exename = read_exename(),
+		.max_stack = 10,
+		.depth = OPT_DEPTH_DEFAULT,
+	};
+	struct uftrace_data handle;
+	struct uftrace_raw_dump dump = {
+		.ops = {
+			.header         = dump_chrome_header,
+			.task_rstack    = dump_chrome_task_rstack,
+			.kernel_func    = dump_chrome_kernel_rstack,
+			.perf_event     = dump_chrome_perf_event,
+			.footer         = dump_chrome_footer,
+		},
+	};
+
+	TEST_EQ(prepare_test_data(&opts, &handle), 0);
+
+	pr_dbg("dump contents in time order\n");
+	do_dump_replay(&dump.ops, &opts, &handle);
+
+	release_test_data(&opts, &handle);
+	return TEST_OK;
+}
+#endif /* UNIT_TEST */
