@@ -1,5 +1,6 @@
 #include <inttypes.h>
 #include <signal.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdio_ext.h>
 #include <stdlib.h>
@@ -14,8 +15,6 @@
 #include "utils/symbol.h"
 #include "utils/utils.h"
 
-#include "libtraceevent/event-parse.h"
-
 static int column_index;
 static int prev_tid = -1;
 
@@ -640,7 +639,7 @@ void get_argspec_string(struct uftrace_task_reader *task, char *args, size_t len
 			}
 
 			dinfo = &map->mod->dinfo;
-			estr = get_enum_string(&dinfo->enums, spec->type_name, (int)val.i);
+			estr = get_enum_string(&dinfo->enums, spec->type_name, val.i);
 			if (strstr(estr, "|") && strcmp("|", color_enum_or)) {
 				struct strv enum_vals = STRV_INIT;
 
@@ -739,7 +738,10 @@ static int print_graph_rstack(struct uftrace_data *handle, struct uftrace_task_r
 		goto out;
 
 	if (rstack->type == UFTRACE_ENTRY) {
-		if (symname[strlen(symname) - 1] != ')' || rstack->more)
+		int len = strlen(symname);
+
+		if (symname[len - 1] != ')' || rstack->more ||
+		    (len > 10 && !strcmp(symname + len - 10, "operator()")))
 			str_mode |= NEEDS_PAREN;
 	}
 
@@ -1177,3 +1179,77 @@ int command_replay(int argc, char *argv[], struct uftrace_opts *opts)
 
 	return ret;
 }
+
+#ifdef UNIT_TEST
+static const char *record_type_str(struct uftrace_record *rec)
+{
+	switch (rec->type) {
+	case UFTRACE_ENTRY:
+		return "ENTRY";
+	case UFTRACE_EXIT:
+		return "EXIT";
+	case UFTRACE_LOST:
+		return "LOST";
+	case UFTRACE_EVENT:
+		return "EVENT";
+	default:
+		break;
+	}
+	return "UNKNOWN";
+}
+
+TEST_CASE(replay_command)
+{
+	struct uftrace_opts opts = {
+		.dirname = "replay-graph-test",
+		.exename = read_exename(),
+		.max_stack = 10,
+		.depth = OPT_DEPTH_DEFAULT,
+	};
+	struct uftrace_data handle;
+	struct uftrace_task_reader *task;
+	uint64_t prev_time = 0;
+
+	TEST_EQ(prepare_test_data(&opts, &handle), 0);
+
+	setup_field(&output_fields, &opts, &setup_default_field, field_table,
+		    ARRAY_SIZE(field_table));
+	if (peek_rstack(&handle, &task) == 0)
+		print_header(&output_fields, "#", "FUNCTION", 1, false);
+	if (!list_empty(&output_fields))
+		pr_out("\n");
+
+	pr_dbg("replay test data in graph format\n");
+	while (read_rstack(&handle, &task) == 0) {
+		struct uftrace_record *rstack = task->rstack;
+		uint64_t curr_time = rstack->time;
+
+		if (!fstack_check_opts(task, &opts)) {
+			pr_dbg("task=%d time=%lu skip\n", task->tid, rstack->time);
+			continue;
+		}
+
+		pr_dbg("task=%d time=%lu depth=%d type=%s\n", task->tid, rstack->time,
+		       rstack->depth, record_type_str(rstack));
+
+		/*
+		 * data sanity check: timestamp should be ordered.
+		 * But print_graph_rstack() may change task->rstack
+		 * during fstack_skip().  So check the timestamp here.
+		 */
+		if (curr_time) {
+			if (prev_time > curr_time)
+				print_warning(task);
+			prev_time = rstack->time;
+		}
+
+		/* this will merge adjacent ENTRY and EXIT */
+		TEST_EQ(print_graph_rstack(&handle, task, &opts), 0);
+	}
+
+	print_remaining_stack(&opts, &handle);
+
+	release_test_data(&opts, &handle);
+	return TEST_OK;
+}
+#endif /* UNIT_TEST */
