@@ -104,6 +104,23 @@ const struct plthook_skip_symbol plt_skip_syms[] = {
 };
 size_t plt_skip_nr = ARRAY_SIZE(plt_skip_syms);
 
+/*
+ * Some compilers generate PLT section even if -fno-plt option is given.
+ * So it cannot simply check if the PLT section is missing for no-plt case.
+ * This is a list of symbols the compilers put in the PLT section regardless
+ * of the option.
+ */
+const char *const noplt_skip_syms[] = {
+	/* For GCC/CLANG */
+	"__stack_chk_fail",
+	"__monstartup",
+	"__cxa_atexit",
+	/* For Rust */
+	"__tls_get_addr",
+	"_Unwind_Resume",
+};
+size_t noplt_skip_nr = ARRAY_SIZE(noplt_skip_syms);
+
 #undef SKIP_SYM
 #undef ALIAS_DECL
 
@@ -118,13 +135,13 @@ static void restore_plt_functions(struct plthook_data *pd)
 
 	for (i = 0; i < dsymtab->nr_sym; i++) {
 		/*
-		 * GOT[0], GOT[1], and GOT[2] are reserved.
+		 * Typically GOT[0], GOT[1], and GOT[2] are reserved.
 		 * GOT[2] initially points to the runtime resolver, but updated
 		 * to plt_hooker for library tracing by uftrace.
 		 * The addresses from GOT[3] are supposed to point the resolved
 		 * addresses for each library function.
 		 */
-		int got_idx = 3 + i;
+		int got_idx = ARCH_PLTGOT_OFFSET + i;
 		bool skipped = false;
 		unsigned long plthook_addr;
 		unsigned long resolved_addr;
@@ -138,8 +155,8 @@ static void restore_plt_functions(struct plthook_data *pd)
 				continue;
 
 			overwrite_pltgot(pd, got_idx, skip_sym->addr);
-			pr_dbg2("overwrite GOT[%d] to %p (%s)\n", got_idx, skip_sym->addr,
-				skip_sym->name);
+			pr_dbg2("overwrite GOT[%d + %d] to %p (%s)\n", i, ARCH_PLTGOT_OFFSET,
+				skip_sym->addr, skip_sym->name);
 
 			skipped = true;
 			break;
@@ -160,8 +177,9 @@ static void restore_plt_functions(struct plthook_data *pd)
 				continue;
 
 			symname = demangle(sym->name);
-			pr_dbg2("restore GOT[%d] from \"%s\"(%#lx) to PLT(base + %#lx)\n", got_idx,
-				symname, resolved_addr, plthook_addr - pd->base_addr);
+			pr_dbg2("restore GOT[%d + %d] from \"%s\"(%#lx) to PLT(base + %#lx)\n", i,
+				ARCH_PLTGOT_OFFSET, symname, resolved_addr,
+				plthook_addr - pd->base_addr);
 			free(symname);
 		}
 		else if (mcount_estimate_return) {
@@ -191,7 +209,12 @@ static int find_got(struct uftrace_elf_data *elf, struct uftrace_elf_iter *iter,
 	bool plt_found = false;
 	unsigned long pltgot_addr = 0;
 	unsigned long plt_addr = 0;
+	unsigned long jmprel_addr = 0;
+	struct uftrace_elf_iter sec_iter;
+	size_t jmprel_nr = 0;
+	size_t jmprel_ent_size = 0;
 	struct plthook_data *pd;
+	const char *fname;
 
 	elf_for_each_shdr(elf, iter) {
 		if (iter->shdr.sh_type == SHT_DYNAMIC)
@@ -204,13 +227,59 @@ static int find_got(struct uftrace_elf_data *elf, struct uftrace_elf_iter *iter,
 			pltgot_addr = (unsigned long)iter->dyn.d_un.d_val + offset;
 			break;
 		case DT_JMPREL:
-			plt_found = true;
+			/*
+			 * Depends on compiler, no-plt binary might have a few plt entries,
+			 * so we need to traverse them.
+			 */
+			jmprel_addr = (uintptr_t)iter->dyn.d_un.d_ptr + offset;
+			break;
+		case DT_PLTRELSZ:
+			jmprel_nr = (unsigned long)iter->dyn.d_un.d_val;
+			break;
+		case DT_RELENT:
+		case DT_RELAENT:
+			jmprel_ent_size = iter->dyn.d_un.d_val;
 			break;
 		default:
 			break;
 		}
 	}
 
+	if (jmprel_ent_size == 0) {
+		pr_dbg("cannot find REL(A)ENT size\n");
+		return 0;
+	}
+
+	elf_for_each_shdr(elf, &sec_iter) {
+		if (sec_iter.shdr.sh_type == SHT_DYNSYM) {
+			elf_get_strtab(elf, &sec_iter, sec_iter.shdr.sh_link);
+			elf_get_secdata(elf, &sec_iter);
+			break;
+		}
+	}
+
+	for (size_t i = 0; i < jmprel_nr; i += jmprel_ent_size) {
+		bool found = false;
+		typeof(sec_iter.rel) *rel = (void *)jmprel_addr + i;
+
+		elf_get_symbol(elf, &sec_iter, elf_rel_symbol(rel));
+		fname = elf_get_name(elf, &sec_iter, sec_iter.sym.st_name);
+
+		/* check if PLT has actual functions other than known symbols */
+		for (size_t k = 0; k < noplt_skip_nr; k++) {
+			if (!strcmp(fname, noplt_skip_syms[k]))
+				found = true;
+		}
+		for (size_t k = 0; k < plt_skip_nr; k++) {
+			if (!strcmp(fname, plt_skip_syms[k].name))
+				found = true;
+		}
+		if (!found) {
+			plt_found = true;
+			break;
+		}
+	}
+
 	if (!plt_found) {
 		pd = mcount_arch_hook_no_plt(elf, modname, offset);
 		if (pd == NULL)
@@ -238,7 +307,7 @@ static int find_got(struct uftrace_elf_data *elf, struct uftrace_elf_iter *iter,
 	pd = xmalloc(sizeof(*pd));
 	pd->mod_name = xstrdup(modname);
 	pd->pltgot_ptr = (void *)pltgot_addr;
-	pd->module_id = pd->pltgot_ptr[1];
+	pd->module_id = pd->pltgot_ptr[ARCH_PLTGOT_MOD_ID];
 	pd->base_addr = offset;
 	pd->plt_addr = plt_addr;
 
@@ -256,7 +325,7 @@ static int find_got(struct uftrace_elf_data *elf, struct uftrace_elf_iter *iter,
 	list_add_tail(&pd->list, &plthook_modules);
 
 	if (plthook_resolver_addr == 0)
-		plthook_resolver_addr = pd->pltgot_ptr[2];
+		plthook_resolver_addr = pd->pltgot_ptr[ARCH_PLTGOT_RESOLVE];
 
 	/*
 	 * BIND_NOW (+ RELRO) makes module id not used and resets to 0.
@@ -264,7 +333,7 @@ static int find_got(struct uftrace_elf_data *elf, struct uftrace_elf_iter *iter,
 	 */
 	if (pd->module_id == 0) {
 		pr_dbg2("update module id to %p\n", pd);
-		overwrite_pltgot(pd, 1, pd);
+		overwrite_pltgot(pd, ARCH_PLTGOT_MOD_ID, pd);
 		pd->module_id = (unsigned long)pd;
 	}
 
@@ -274,7 +343,7 @@ static int find_got(struct uftrace_elf_data *elf, struct uftrace_elf_iter *iter,
 
 	restore_plt_functions(pd);
 
-	overwrite_pltgot(pd, 2, plt_hooker);
+	overwrite_pltgot(pd, ARCH_PLTGOT_RESOLVE, plt_hooker);
 
 	if (getenv("LD_BIND_NOT"))
 		plthook_no_pltbind = true;
@@ -709,7 +778,7 @@ static void update_pltgot(struct mcount_thread_data *mtdp, struct plthook_data *
 		pthread_mutex_lock(&resolver_mutex);
 #endif
 		if (!pd->resolved_addr[dyn_idx]) {
-			int got_idx = 3 + dyn_idx;
+			int got_idx = ARCH_PLTGOT_OFFSET + dyn_idx;
 			plthook_addr = mcount_arch_plthook_addr(pd, dyn_idx);
 			setup_pltgot(pd, got_idx, dyn_idx, (void *)plthook_addr);
 		}
