@@ -655,14 +655,15 @@ static int parse_option(struct uftrace_opts *opts, int key, char *arg)
 
 	case 'k':
 		opts->kernel = true;
+		opts->kernel_depth = 1;
 		break;
 
 	case 'K':
 		opts->kernel = true;
 		opts->kernel_depth = strtol(arg, NULL, 0);
 		if (opts->kernel_depth < 1 || opts->kernel_depth > 50) {
-			pr_use("invalid kernel depth: %s (ignoring...)\n", arg);
-			opts->kernel_depth = 0;
+			pr_use("invalid kernel depth: %s. Set depth to 1.\n", arg);
+			opts->kernel_depth = 1;
 		}
 		break;
 
@@ -1419,31 +1420,35 @@ int main(int argc, char *argv[])
 
 	switch (parse_options(argc, argv, &opts)) {
 	case -1:
-		return 0;
+		ret = 0;
+		goto cleanup;
 	case -2:
 		pr_out(uftrace_usage);
 		pr_out(uftrace_footer);
-		return 0;
+		ret = 0;
+		goto cleanup;
 	case -3:
 		if (opts.use_pager)
 			start_pager(setup_pager());
 		pr_out(uftrace_usage);
 		pr_out(uftrace_help);
 		wait_for_pager();
-		return 0;
+		ret = 0;
+		goto cleanup;
 	}
 
 	if (opts.opt_file)
 		parse_opt_file(&argc, &argv, opts.opt_file, &opts);
 
-	if (opts.exename == NULL) {
+	if (opts.exename == NULL && !opts.list_event) {
 		switch (opts.mode) {
 		case UFTRACE_MODE_RECORD:
 		case UFTRACE_MODE_LIVE:
 		case UFTRACE_MODE_INVALID:
 			pr_out(uftrace_usage);
 			pr_out(uftrace_footer);
-			return 1;
+			ret = 1;
+			goto cleanup;
 		}
 	}
 
@@ -1562,6 +1567,7 @@ int main(int argc, char *argv[])
 
 	wait_for_pager();
 
+cleanup:
 	if (opts.logfile)
 		fclose(logfp);
 
