@@ -28,6 +28,9 @@
 #ifndef EM_AARCH64
 #define EM_AARCH64 183
 #endif
+#ifndef EM_RISCV
+#define EM_RISCV 243
+#endif
 
 /* (global) symbol for kernel */
 static struct uftrace_module kernel;
@@ -472,10 +475,11 @@ static int load_dyn_symbol(struct uftrace_symtab *dsymtab, int sym_idx, unsigned
 	return 1;
 }
 
-static void sort_dynsymtab(struct uftrace_symtab *dsymtab)
+void sort_dynsymtab(struct uftrace_symtab *dsymtab)
 {
 	unsigned i, k;
-
+	if (dsymtab->nr_sym == 0)
+		return;
 	dsymtab->nr_alloc = dsymtab->nr_sym;
 	dsymtab->sym = xrealloc(dsymtab->sym, dsymtab->nr_sym * sizeof(*dsymtab->sym));
 
@@ -595,6 +599,9 @@ int load_elf_dynsymtab(struct uftrace_symtab *dsymtab, struct uftrace_elf_data *
 	else if (elf->ehdr.e_machine == EM_X86_64) {
 		plt_entsize = 16; /* lld (of LLVM) seems to miss setting it */
 	}
+	else if (elf->ehdr.e_machine == EM_RISCV) {
+		plt_addr += 16; /* RISCV64 PLT0 size is 32 */
+	}
 
 	prev_addr = plt_addr;
 	if (found_pltsec)
@@ -643,24 +650,6 @@ out:
 	return ret;
 }
 
-static int load_dynsymtab(struct uftrace_symtab *dsymtab, const char *filename,
-			  unsigned long offset, unsigned long flags)
-{
-	int ret;
-	struct uftrace_elf_data elf;
-
-	if (elf_init(filename, &elf) < 0) {
-		pr_dbg("error during open symbol file: %s: %m\n", filename);
-		return -1;
-	}
-
-	pr_dbg3("loading dynamic symbols from %s (offset: %#lx)\n", filename, offset);
-	ret = load_elf_dynsymtab(dsymtab, &elf, offset, flags);
-
-	elf_finish(&elf);
-	return ret;
-}
-
 static void merge_symtabs(struct uftrace_symtab *left, struct uftrace_symtab *right)
 {
 	size_t nr_sym = left->nr_sym + right->nr_sym;
@@ -714,6 +703,26 @@ static void merge_symtabs(struct uftrace_symtab *left, struct uftrace_symtab *ri
 	left->name_sorted = true;
 }
 
+static int load_dynsymtab(struct uftrace_symtab *dsymtab, const char *filename,
+			  unsigned long offset, unsigned long flags)
+{
+	struct uftrace_symtab dsymtab_noplt = {};
+	struct uftrace_elf_data elf;
+
+	if (elf_init(filename, &elf) < 0) {
+		pr_dbg("error during open symbol file: %s: %m\n", filename);
+		return -1;
+	}
+
+	pr_dbg3("loading dynamic symbols from %s (offset: %#lx)\n", filename, offset);
+	load_elf_dynsymtab(dsymtab, &elf, offset, flags);
+	arch_load_dynsymtab_noplt(&dsymtab_noplt, &elf, offset, flags);
+	merge_symtabs(dsymtab, &dsymtab_noplt);
+
+	elf_finish(&elf);
+	return dsymtab->nr_sym;
+}
+
 static int update_symtab_using_dynsym(struct uftrace_symtab *symtab, const char *filename,
 				      unsigned long offset, unsigned long flags)
 {
