@@ -302,6 +302,14 @@ static inline bool host_is_lp64(void)
 	return get_elf_class() == ELFCLASS64;
 }
 
+static inline const char *get_endian_str(void)
+{
+	if (get_elf_endian() == ELFDATA2LSB)
+		return "LE";
+	else
+		return "BE";
+}
+
 static inline char *has_kernel_opt(char *buf)
 {
 	int idx = 0;
@@ -397,7 +405,6 @@ void free_parsed_cmdline(char **argv);
 
 struct uftrace_data;
 
-char *get_event_name(struct uftrace_data *handle, unsigned evt_id);
 char *absolute_dirname(const char *path, char *resolved_path);
 
 char *uftrace_strerror(int errnum, char *buf, size_t buflen);
@@ -410,6 +417,11 @@ void stacktrace(void);
 #define TRAP() raise(SIGTRAP)
 #endif
 
+#define DTRAP()                                                                                    \
+	if (DEBUG_MODE) {                                                                          \
+		TRAP();                                                                            \
+	}
+
 #define ASSERT(cond)                                                                               \
 	if (unlikely(!(cond))) {                                                                   \
 		pr_red("%s:%d: %s: ASSERT `%s' failed.\n", __FILE__, __LINE__, __func__, #cond);   \
